/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * a table to map between remote list commands to our menus
 */


#include "rli.h"

struct t_cmd_table_R {
  int lii;		/*list index*/
  int lim;		/*message number to list*/
  int prompt;		/*index to string resource of prompts*/
  int dups;		/*number of duplicate commands*/
  int opendoc;		/*needs to open document */
  int want_busy;	/*don't do it if busy*/
};

typedef struct t_cmd_table_R t_cmd_table,*t_cmd_table_pt;

#define ISBUSY (TRUE)

#define NOBUSY (FALSE)


t_cmd_table folder_table[]={
  {FOLDER_RL, 0,   0,1,0,ISBUSY},    /* fl_create */
  {FOLDER_RL, 1,  -1,1,0,ISBUSY},    /* fl_delete */
  {FOLDER_RL, 2,   1,1,0,ISBUSY},    /* fl_rename */
  {FOLDER_RL, 3,   2,1,0,ISBUSY},    /* fl_merge */
  {FOLDER_RL, 4,  -1,1,0,ISBUSY},    /* fl_info */
  {FOLDER_RL, 5,  -1,1,0,ISBUSY},    /* fl_seen */
  {FOLDER_RL, 6,  -1,0,0,NOBUSY},    /* fl_subscribe_full */
  {FOLDER_RL, 7,  -1,0,0,NOBUSY},    /* fl_subscribe_none */
  {FOLDER_RL, 8,  -1,0,0,NOBUSY},    /* fl_show_all */
  {FOLDER_RL, 9,  -1,0,0,NOBUSY},    /* fl_subscribe_ask */
  {FOLDER_RL,10,   3,0,0,ISBUSY},    /* fl_expose_match */
  {FOLDER_RL,11,  -1,0,0,ISBUSY},    /* fl_expose_personal */
  {FOLDER_RL,12,  -1,0,0,ISBUSY},    /* fl_expose_all */
  {FOLDER_RL,13,  -1,0,0,ISBUSY},    /* fl_expose_subscribed */
  {FOLDER_RL,14,  -1,0,0,ISBUSY},    /* fl_expose_changed */
  {FOLDER_RL,15,  -1,0,0,ISBUSY},    /* fl_subscribe_print */
  {FOLDER_RL,16,  -1,1,1,ISBUSY},    /* fl_post */
  {FOLDER_RL,17,  -1,0,0,ISBUSY},    /* fl_empty_mailbox */
  {FOLDER_RL,18,  -1,0,0,ISBUSY},    /* fl_punt */
  {FOLDER_RL,19,  10,0,0,ISBUSY}     /* fl_expose_name */
};

int folder_table_size=(sizeof(folder_table)/sizeof(t_cmd_table));

t_cmd_table caption_table[]={
  {CAPTION_RL, 0, -1,1,0,ISBUSY},    /* cp_KillAndNext */
  {CAPTION_RL, 1, -1,1,0,ISBUSY},    /* cp_NextMsg */
  {CAPTION_RL, 2, -1,1,0,ISBUSY},    /* cp_PrevMsg */
  {CAPTION_RL, 3, -1,1,0,NOBUSY},    /* cp_DeleteMsg */
  {CAPTION_RL, 4, -1,1,0,NOBUSY},    /* cp_UndeleteMsg */
  {CAPTION_RL, 5,  4,0,0,NOBUSY},    /* cp_FileInto */
  {CAPTION_RL, 6, -1,1,0,ISBUSY},    /* cp_ShowHeaders */
  {CAPTION_RL, 7, -1,1,1,ISBUSY},    /* cp_RestoreDraft */
  {CAPTION_RL, 8, -1,1,0,NOBUSY},    /* cp_SetNew */
  {CAPTION_RL, 9, -1,0,1,ISBUSY},    /* cp_ReplySender */
  {CAPTION_RL,10, -1,0,1,ISBUSY},    /* cp_ReplyReader */
  {CAPTION_RL,11,  5,0,0,ISBUSY},    /* cp_Resend */
  {CAPTION_RL,12, -1,0,1,ISBUSY},    /* cp_Forward */
  {CAPTION_RL,13, -1,0,1,ISBUSY},    /* cp_ReplyAll */
  {CAPTION_RL,14, -1,0,1,ISBUSY},    /* cp_Post */
  {CAPTION_RL,15, -1,0,0,ISBUSY},    /* cp_PrintMessage */
  {CAPTION_RL,16,  9,0,0,ISBUSY},    /* cp_SaveAsFile */
  {CAPTION_RL,17, -1,0,0,ISBUSY},    /* cp_rot */
  {CAPTION_RL,18, -1,0,0,NOBUSY}     /* cp_expunge */
};

int caption_table_size=(sizeof(caption_table)/sizeof(t_cmd_table));

t_cmd_table body_table[]={
  {BODY_RL, 0, -1,1,0,ISBUSY},	    /* mr_KillAndNext */
  {BODY_RL, 1, -1,1,0,ISBUSY},	    /* mr_NextMsg */
  {BODY_RL, 2, -1,1,0,ISBUSY},	    /* mr_PrevMsg */
  {BODY_RL, 3, -1,1,0,NOBUSY},	    /* mr_Delete */
  {BODY_RL, 4, -1,1,0,NOBUSY},	    /* mr_Undelete */
  {BODY_RL, 5,  4,0,0,ISBUSY},	    /* mr_FileInto */
  {BODY_RL, 6, -1,1,0,ISBUSY},	    /* mr_ShowHeaders */
  {BODY_RL, 7, -1,1,1,ISBUSY},	    /* mr_RestoreDraft */
  {BODY_RL, 8, -1,1,0,ISBUSY},	    /* mr_Setnew */
  {BODY_RL, 9, -1,0,1,ISBUSY},	    /* mr_ReplySender */
  {BODY_RL,10, -1,0,1,ISBUSY},	    /* mr_ReplyReaders */
  {BODY_RL,11,  5,0,0,ISBUSY},	    /* mr_Resend */
  {BODY_RL,12, -1,0,1,ISBUSY},	    /* mr_Forward */
  {BODY_RL,13, -1,0,1,ISBUSY},	    /* mr_ReplyAll */
  {BODY_RL,14, -1,0,1,ISBUSY},	    /* mr_Post */
  {BODY_RL,15, -1,0,0,ISBUSY},	    /* mr_PrintMessage */
  {BODY_RL,16,  9,0,0,ISBUSY},	    /* mr_SaveAsFile */
  {BODY_RL,17, -1,0,0,ISBUSY},	    /* mr_rot */
  {BODY_RL,18, -1,0,0,NOBUSY},      /* mr_expunge */
  {BODY_RL,19, -1,0,0,NOBUSY},      /* mr_search */
  {BODY_RL,20, -1,0,1,ISBUSY}       /* mr_draft_revive */
};

int body_table_size=(sizeof(body_table)/sizeof(t_cmd_table));

t_cmd_table send_table[]={
  {SEND_RL, 0, -1,1,0,ISBUSY},	    /* sm_Send */
  {SEND_RL, 1, -1,1,0,ISBUSY},	    /* sm_Verify */
  {SEND_RL, 2,  6,0,0,NOBUSY},	    /* sm_Whois */
  {SEND_RL, 3, -1,0,1,NOBUSY},	    /* sm_new */
  {SEND_RL, 4,  7,1,0,ISBUSY},	    /* sm_append */
  {SEND_RL, 5, -1,0,0,NOBUSY},	    /* sm_set_preferences */
  {SEND_RL, 6, -1,0,0,NOBUSY},	    /* sm_DisplayMotd */
  {SEND_RL, 7,  8,0,1,ISBUSY},	    /* sm_OpenRemoteDraft */
  {SEND_RL, 8, -1,0,0,NOBUSY},	    /* sm_quit */
  {SEND_RL, 9, -1,0,0,NOBUSY},	    /* sm_console_banner */
  {SEND_RL,10, -1,0,0,NOBUSY},	    /* sm_forwarding address */
  {SEND_RL,11, -1,0,0,NOBUSY},	    /* sm_change_password */
  {SEND_RL,12,  6,0,0,NOBUSY},	    /* sm_finger */
  {SEND_RL,13,  0,0,0,NOBUSY},	    /* sm_ls */
  {SEND_RL,14,  0,0,0,NOBUSY},	    /* sm_cat */
  {SEND_RL,15,  0,0,0,NOBUSY},	    /* sm_rm */
  {SEND_RL,16, 12,0,0,NOBUSY},	    /* sm_PrintQueue */
  {SEND_RL,17, 13,0,0,NOBUSY}	    /* sm_AndrewHelp */
};

int send_table_size=(sizeof(send_table)/sizeof(t_cmd_table));

t_cmd_table help_table[]={
  {HELP_RL, 0, 0,0,0,NOBUSY}	    /* Bring Help */
};

int help_table_size=(sizeof(help_table)/sizeof(t_cmd_table));


t_cmd_table dir_table[]={
  {DIRECTORY_RL,  0, -1,0,0,NOBUSY},	/* dr_remove */
  {DIRECTORY_RL,  1, 14,0,0,NOBUSY},	/* dr_rename */
  {DIRECTORY_RL,  2, -1,0,1,ISBUSY},	/* dr_edit */
  {DIRECTORY_RL,  3, -1,0,0,NOBUSY},	/* dr_show */
  {DIRECTORY_RL,  4, -1,0,0,NOBUSY},	/* dr_get */
  {DIRECTORY_RL,  5, -1,0,0,NOBUSY},	/* dr_put */
  {DIRECTORY_RL,  6, -1,0,0,NOBUSY},	/* dr_home */
  {DIRECTORY_RL,  7, -1,0,0,NOBUSY},	/* dr_open_as_draft */
  {DIRECTORY_RL,  8, 15,0,0,NOBUSY},	/* dr_mkdir */
  {DIRECTORY_RL,  9, -1,0,0,NOBUSY},	/* dr_update */
  {DIRECTORY_RL, 10, -1,0,0,NOBUSY},	/* dr_close */
  {DIRECTORY_RL, 11, -1,0,0,ISBUSY},	/* dr_long */
  {DIRECTORY_RL, 12, 16,0,0,NOBUSY},	/* dr_cd */
  {DIRECTORY_RL, 13, -1,0,1,ISBUSY},	/* dr_quickedit */
  {DIRECTORY_RL, 14, -1,0,0,NOBUSY},	/* dr_quota */
  {DIRECTORY_RL, 15, -1,0,0,NOBUSY},    /* dr_info */
  {DIRECTORY_RL, 16, 17,0,1,ISBUSY}     /* dr_new */
};

int dir_table_size=(sizeof(dir_table)/sizeof(t_cmd_table));

int GetRemoteCommandEntry(int  idx, int listnum,long *strnum, long *instances, 
			  long *needfile,int  *isbusy)
{
	int atbl_size;
	t_cmd_table_pt atbl;

	switch (listnum) {
		case FOLDER_RL:
			atbl = folder_table;
			atbl_size = folder_table_size;
			break;
		case CAPTION_RL:
			atbl = caption_table;
			atbl_size = caption_table_size;
			break;
		case BODY_RL:
			atbl = body_table;
			atbl_size = body_table_size;
			break;
		case SEND_RL:
			atbl = send_table;
			atbl_size = send_table_size;
			break;
		case HELP_RL:
			atbl = help_table;
			atbl_size = help_table_size;
			break;
		case DIRECTORY_RL:
			atbl = dir_table;
			atbl_size = dir_table_size;
			break;
		default:
			return FALSE;
	}

	if (idx < 0 || idx >= atbl_size) return FALSE;
	atbl+=idx;

	*strnum = atbl->prompt;
	*instances = atbl->dups;
	*needfile = atbl->opendoc;
	*isbusy = atbl->want_busy;
	return TRUE;
}

