/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <mmsclie.h>
#include <mmclib.h>

t_catch_rl catch_rl;

extern int sys_nerr;
extern char *sys_errlist[];

/*
 * convert an errno to text
 */
static char *errno_to_text(anerr)
int anerr;
{
    static char buf[30];
    if((anerr<=0)||
	(anerr>sys_nerr)) {
	if(anerr==0)return "snone";
	sprintf(buf,"serr%d",anerr);
	return buf;
    }
    else
	return sys_errlist[anerr];
}

void report_console(
	 fmt,
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10)
char *fmt;
long a1,a2,a3,a4,a6,a7,a8,a9,a10;
long b1,b2,b3,b4,b6,b7,b8,b9,b10;
long c1,c2,c3,c4,c6,c7,c8,c9,c10;
{
  char buf[255];
  sprintf(buf, fmt, 
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10);
  CON_WriteLine(buf);
}

void rl_error(fmt,
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10)
char *fmt;
long a1,a2,a3,a4,a6,a7,a8,a9,a10;
long b1,b2,b3,b4,b6,b7,b8,b9,b10;
long c1,c2,c3,c4,c6,c7,c8,c9,c10;
{
  int save_errno = errno;

#ifdef IBMPC
  if (save_errno == EXDEV) save_errno = 0; /* bogus msc error */
#else
  if (save_errno == EINTR) save_errno = 0; /* interrupted system call */
#endif

  report_console(fmt, 
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10);

  if(save_errno > 0)
    report_console("last errno=%d '%s'",save_errno,errno_to_text(save_errno));

  close_cleanup();
  clear_rl_catch();

#ifdef DESQview
  DV_end_critical();
#endif

  longjmp(&catch_rl.catch[catch_rl.inuse], 1);
}

void rl_parerr(ioblk,
	 fmt,
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10)
long ioblk;
char *fmt;
long a1,a2,a3,a4,a6,a7,a8,a9,a10;
long b1,b2,b3,b4,b6,b7,b8,b9,b10;
long c1,c2,c3,c4,c6,c7,c8,c9,c10;
{
 errno = 0;
 rl_error(fmt,
	 a1,a2,a3,a4,a6,a7,a8,a9,a10,
	 b1,b2,b3,b4,b6,b7,b8,b9,b10,
	 c1,c2,c3,c4,c6,c7,c8,c9,c10);
}
