/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * portable packet packing/unpacking interface
 * unix io module implementation
 */

#include <mmspfio.h>

/*
 * recieve a connection, data or console
 */

void recv_connect(me)
pfconn_pt me;
{
    struct sockaddr_in sin;
    int sinlen;
    int newfd;
    sinlen=sizeof(sin);
    er_ucall((newfd=accept(me->fd,&sin,&sinlen)),("accept"));
    me->fd=newfd;
}

/*
 * enable signal bits for connected connection
 */
int serv_any_enable(me,si)
pfconn_pt me;
select_info_pt si;
{
  FD_SET(me->fd, &si->read_fd); /* set this any way */
  if(serv_data_sel(me))		/*read what we can*/
    FD_SET(me->fd,&si->read_fd); /*perhaps have room to read still more*/
  if(serv_write_sel(me))	/*write what we can*/
    FD_SET(me->fd,&si->write_fd); /*perhaps want to write still more*/
  return 0;
}

/*
 * during connection need read
 */
int serv_conn_enable(me,si)
pfconn_pt me;
select_info_pt si;
{
  FD_SET(me->fd,&si->read_fd);
  return 0;
}

/*
 * accept a server data connection
 */
int never_enable(me,si)
pfconn_pt me;
select_info_pt si;
{
  return 0;
}
/*
 * initilize a select info block
 */
void mn_select_init(si)
select_info_pt si;
{
#define PLUS_INF (0x7FFFFFFFL)

  bzero(si,sizeof(*si));
  si->timeout.tv_sec=PLUS_INF;
}

/*
 * select routine that should never be selected
 */
int never_sel(me)
pfconn_pt me;
{
  rl_error("bogus select fd=%d",me->fd);
  return 0;
}

/*
 * action table to read a packets data
 */
int serv_data_sel();
int serv_write_sel();
int serv_any_enable();
pfio_actions serv_data={
  serv_any_enable,
  serv_data_sel,
  serv_write_sel,
  never_sel,
};

int serv_conn_sel(me)
pfconn_pt me;
{
  recv_connect(me);
  set_no_block(me->fd);
  me->actor= &serv_data;
  return 0;
}

/*
 * send to our partner
 * returns true if there is more data to transmit
 * and we want to be woken up when it can be
 */

extern long bytes_transmitted;

int serv_write_sel(me)
pfconn_pt me;
{
  char *xmit_data;
  int xmit_len;
  while(OBUF_get_xmit(&xmit_data,&xmit_len)) {
    register iores=write(me->fd,xmit_data,xmit_len);
    if(iores>0) {
      OBUF_report_xmit(iores);
      bytes_transmitted += iores;
      CON_SetBusy(TRUE);
      clear_keepalive_timer();
    }
    else if(errno==EWOULDBLOCK) /*would block?*/
      return TRUE;
    else
      rl_error("write failed");
  }
  return FALSE;			/*no more data to transmit*/
}

/*
 * read data available
 */
int serv_data_sel(me)
pfconn_pt me;
{
  register int rc;
  char *buf;
  int len;
  while(IBUF_get_buf(&buf,&len)) {
    rc=read(me->fd,buf,len);	/*read what we can*/
    if (rc == 0) rl_error("connection died");
    if(rc == -1) {
      if(errno==EWOULDBLOCK)return TRUE; /*nothing there*/
      er_ucall(rc,("read"));	/*make sure its not an error code*/
    }
    recieved_bytes+=rc;		/*record for idle/ack*/
    IBUF_report_recv(rc);	/*report how much data we read*/
  }
  return FALSE;			/*no data left to transmit*/
}

/*
 * see if the select was due to us
 */
int mn_select_p(si)
select_info_pt si;
{
  PF_FOR_ALL_CONS(cn)
    if(cn->fd < 0)continue;
    if(FD_ISSET(cn->fd,&si->read_fd))
      (*cn->actor->read_select)(cn);
    if(FD_ISSET(cn->fd,&si->write_fd))
      (*cn->actor->write_select)(cn);
    if(FD_ISSET(cn->fd,&si->except_fd))
      (*cn->actor->except_select)(cn);
  PF_END_FOR_ALL_CONS
}

/*
 * give each connection a chance to enable some wait bits
 */
void mn_select_set(si)
select_info_pt si;
{
  PF_FOR_ALL_CONS(cn)
    if (cn->fd >= 0)
      (*cn->actor->enable)(cn,si);
  PF_END_FOR_ALL_CONS
}


