/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * portable packet packing/unpacking interface
 * unix io module interface
 * by Aaron Wohl
 */

#ifndef MMSPFIO_H
#define MMSPFIO_H

#include "pfiou.h"

#include <sys/time.h>

typedef int (*pfio_rtn)();

struct pfio_actions_R {
  pfio_rtn enable;		/*chance to turn on select bits*/
  pfio_rtn read_select;		/*if our fd is selected for read*/
  pfio_rtn write_select;		/*if our fd is selected for write*/
  pfio_rtn except_select;		/*if our fd is selected for an exception*/
};
typedef struct pfio_actions_R pfio_actions,*pfio_actions_pt;

/*
 * a connection
 */
struct pfconn_R {
  int fd;			/*connections fd or zero*/
  pfio_actions_pt actor;	/*current actor*/
};
typedef struct pfconn_R pfconn,*pfconn_pt;

extern pfconn pf_all_conns[];	/*table of all connections*/
extern int pf_all_size;		/*number of table entries*/

#define PF_FOR_ALL_CONS(xx_cn) \
  { \
    register int pfxxi; \
    register pfconn_pt xx_cn; \
    for(pfxxi=0,xx_cn=pf_all_conns;pfxxi<pf_all_size;pfxxi++,xx_cn++) {

#define PF_END_FOR_ALL_CONS } }


/*
 * select information block
 */
struct select_info_R {
  struct timeval timeout;
  fd_set read_fd;
  fd_set write_fd;
  fd_set except_fd;
  int nfound;
};
typedef struct select_info_R select_info,*select_info_pt;

#define er_ucall(xx_ures,xx_err){if((xx_ures)<0){rl_error xx_err ;};}

int never_enable();
void recv_connect();
int serv_any_enable();
int serv_conn_enable();
void mn_select_init();
int never_sel();
int serv_conn_sel();
int serv_write_sel();
int serv_data_sel();
 int mn_select_p();
void get_local_addr();
void mn_select_set();
extern pfio_actions serv_data;

extern long recieved_bytes;

#ifndef FD_SET
#include <fddefs.h>
#endif /* FD_SET */

#ifndef IBMPC
extern int errno;
#endif /* IBMPC */


#endif /* MMSPFIO_H */
