/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <string.h>

#include "password.h"
#include "dialog.h"
#include "xmalloc.h"
#include "ezconfig.h"

typedef struct passwordlist_s {
    char *hosts[10];
    char *passwords[10];

    int size;

} passwordlist_t;

passwordlist_t *gpasslist = NULL;

static char *
find_password(passwordlist_t *list, const char *server)
{
    int i;

    for (i = 0; i < list->size; i++)
	if (strcmp(list->hosts[i],server)==0)
	    return list->passwords[i];

    return NULL;
}

/* JCS - clear a password from the password list */
void
password_purge(const char *server)
{
  int 
    i;

  if ( gpasslist == NULL )
    return;

  for ( i = 0; i < gpasslist->size; i++ )
    if ( strcmp ( gpasslist->hosts[i], server ) == 0 ) {
      free ( gpasslist->passwords[i] );
      free ( gpasslist->hosts[i] );
      gpasslist->passwords[i] = NULL;
      gpasslist->hosts[i]     = NULL;
      for ( i++; i < gpasslist->size; i++ ) { 
        gpasslist->passwords[i-1] = gpasslist->passwords[i];
        gpasslist->hosts[i-1]     = gpasslist->hosts[i];
      }
      gpasslist->size--;
      return;
    }      
}

static void
set_password(passwordlist_t *list, const char *server, char *pass)
{
    if (list->size == 10) return;

    list->hosts[list->size] = xstrdup(server);
    list->passwords[list->size] = pass;
    list->size++;
}

int password_get(const char *server, char **password)
{
    char prompt[256];
    char *pass;
    int r;

    if (gpasslist == NULL) {
	
	gpasslist = xmalloc(sizeof(passwordlist_t));
	memset(gpasslist, 0, sizeof(passwordlist_t));

    } else {
	pass = find_password(gpasslist, server);
	if (pass) {
	    *password = pass;
	    return 0;
	}
    }
    
    pass = xmalloc(256);
    pass[0] = '\0';

    snprintf(prompt,sizeof(prompt)-1,"Enter your password for %s",server);

    r = DLG_GetPasswordFromUser(prompt,pass);

    if (r == TRUE) {
	if (config_getswitch("RememberPasswords",0))
	    set_password(gpasslist,server,pass);

	*password = pass;
	return 0;
    }
	
    return -1;
}
