/* 
 * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The name "Carnegie Mellon University" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For permission or any other legal
 *    details, please contact  
 *      Office of Technology Transfer
 *      Carnegie Mellon University
 *      5000 Forbes Avenue
 *      Pittsburgh, PA  15213-3890
 *      (412) 268-4387, fax: (412) 268-7395
 *      tech-transfer@andrew.cmu.edu
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Computing Services
 *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
 *
 * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
 * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * remote list both server and client shared
 * routines
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>

#include "rli.h"

extern RL_header RL_all[];

/*
 * number of output buffers
 */
#ifndef NUM_OBUFS
#define NUM_OBUFS (9)
#endif
#ifndef OBUF_minbuf
#define OBUF_minbuf (2)
#endif

struct input_handler_R {
    int (*room)();		/*say now much room is available*/
    void (*take_this)();		/*some data was read*/
    int (*feed_me)();		/*return next input buffer*/
};
typedef struct input_handler_R input_handler,*input_handler_pt;

/*
 * forward declaration due to circular refrence
 */
extern input_handler len_actor;	/*read a packets length*/

/*
 * a buffer this big can recieve any packet
 */
#define MAX_PAK_SIZE (3500)

struct OUTGOING_R {
    char outbuf[MAX_PAK_SIZE];	/*buffer for transmiting data*/
};
typedef struct OUTGOING_R OUTGOING,*OUTGOING_pt;

typedef OUTGOING t_bufs[NUM_OBUFS];	/*output buffers*/

/*
 * remote list shared (server/client implementation)
 * globals
 */
#define L_BUF_SIZE (((long)MAX_PAK_SIZE)*((long)NUM_OBUFS))

struct t_rgl_R {
    int we_are_dead;		/*if network is dead*/
    int	urgent_mode;		/* we are in urgent mode(blocked) */
    int	urgent_deep;		/* stack up the urgents */
    t_bufs *bufs;		/*output buffers*/
    int ready_bufs;		/*number of buffers ready to transmit*/
    int insi;			/*insertion pointer (index into bufs)*/
    PFM ins;			/*controls for writting queueing output*/
    PFM remv;			/*controls for dequeueing and transmiting*/

    char inbuf[MAX_PAK_SIZE];	/*input buffer*/
    input_handler_pt iactor;	/*actor to handle input*/
    PFM imsg;			/*input message parse block*/
};
typedef struct t_rgl_R t_rgl;

t_rgl rgl;

/*
 * is the output queue in a bad way
 */
int output_queue_is_full(void)
{
  return (rgl.ready_bufs>=NUM_OBUFS);
}

/*
 * is there a valid network connection
 */
int is_net_dead(void)
{
    return(rgl.we_are_dead ? TRUE : FALSE);
}

void set_net_dead(int aval)
{
    rgl.we_are_dead=aval;
}

#ifdef RECURSSIVE_EVENTS
int we_are_urgent()
{
    return rgl.urgent_mode;
}
#endif /* RECURSSIVE_EVENTS */

#ifdef RECURSSIVE_EVENTS
void set_urgent_mode(mode)
int mode;
{
    if (mode == TRUE) {
	rgl.urgent_deep++;
    } else {
	rgl.urgent_deep--;
    }
    rgl.urgent_mode = mode;

    if (rgl.urgent_deep < 0) rl_error("Urgent stack corrupted");	
}
#endif /* RECURSSIVE_EVENTS */

#ifdef RECURSSIVE_EVENTS
void check_urgent_depth()
{
    if (rgl.urgent_deep > 0)
	rgl.urgent_mode = TRUE;
}
#endif /* RECURSSIVE_EVENTS */

/*
 * comput an index offset from a base offset
 */
int WRAP_OINDEX(index)
int index;
{
    if(index<0)index=NUM_OBUFS - abs(index);
    if(index>=NUM_OBUFS)index-=NUM_OBUFS;
    return index;
}

/***********************************************************************
 *								       								   *
 *                input buffer management			         		   *
 *								       								   *
 ***********************************************************************/

#define rl_parerr 4



void rlbi_initialize()
{
  static int RLBI_has_initialized = FALSE;
  t_bufs *saved_t_bufs = rgl.bufs;

  bzero(&rgl,sizeof(rgl));
  rgl.we_are_dead=TRUE;

  if (RLBI_has_initialized && saved_t_bufs)
    rgl.bufs = saved_t_bufs;
  else {
    rgl.bufs=(t_bufs *)c_get_mem(sizeof(*rgl.bufs));
    RLBI_has_initialized = TRUE;
  }
}

/*
 * return a buffer to recieve incomming data if
 * there is one available
 */
int IBUF_get_buf(buf,len)
char **buf;
int *len;
{
    return (*(rgl.iactor->room))(buf,len);
}

/*
 * report that some data was filled in to the
 * buffer IBUF_get_buf made available
 */
void IBUF_report_recv(len)
int len;
{
    (*(rgl.iactor->take_this))(len);
}

/*
 * return a message out of the input buffer
 * return TRUE if there was one available
 */
int IBUF_feed_me(m)
PFM_pt m;
{
  
  return (*(rgl.iactor->feed_me))(m);
}


/*
 * actor routines for reading the two byte length on the front
 * of a packet
 */



/*
 * return a message from the input stream
 */



/*
 * actor to read in the body of a message
 */

/*
 * say how much data should be read
 */



/*
 * say how much data should be read
 */


/*
 * accept a report of how much data was read
 */


/*
 * return a message from the input stream
 */







/***********************************************************************								       *								      								   *
 *                output buffer management						       *
 *					  											       *
 ***********************************************************************/

/*
 * start output of the next available buffer
 */
static void obuf_start(void)
{
    if(rgl.ready_bufs==0)
	return;
  /*
   * transmit data from here
   */

    rgl.remv.initial=(uint8 *)rgl.bufs + WRAP_OINDEX(rgl.insi-rgl.ready_bufs) * MAX_PAK_SIZE;
    rgl.remv.buf=rgl.remv.initial;
    rgl.remv.limit=rgl.remv.buf+ (int)(PFM_FETCH16(rgl.remv.buf));    
}

/*
 * return a buffer for transmiting if there is one
 */
int OBUF_get_xmit(buf,len)
char **buf;
int *len;
{
    (*len)=0;			/*assume nothing to transmit*/
    if(rgl.remv.buf==NIL)		/*output buffer active?*/
	obuf_start();		/*make one so if possible*/
    if(rgl.remv.buf==NIL)		/*output buffer active?*/
	return FALSE;		/*no, so no output*/
    (*len)=PFM_remaining(rgl.remv); /*bytes remaining to transmit*/
    (*buf)=rgl.remv.buf;		/*from here*/
    return TRUE;
}

/*
 * some of the data that OBUF_get_xmit asked to
 * to be transmited actually was
 */
void OBUF_report_xmit(len)
int len;
{
    int remaining;

    rgl.remv.buf+=len;
    remaining=PFM_remaining(rgl.remv);
    if(remaining<0)
	rl_error("more data was transmited than existed %d");
    if(remaining>0)return;
    /* this buffer was finished off */
    rgl.remv.buf=NIL;
    rgl.ready_bufs--;
}

/*
 * nothing to do,or buffer is full
 * any queued data will be transmited
 */
void OBUF_flush()
{
    if(rgl.ins.buf==NIL)		/*have an active output buffer*/
	return;			/*no, done*/
    /* patch in the true length*/
    PFM_PUT16(rgl.ins.initial,BYTES_IN_BUF(rgl.ins));
    rgl.ready_bufs++;
    rgl.insi++;
    rgl.insi=WRAP_OINDEX(rgl.insi);
    rgl.ins.buf=NIL;
}


/*
 * start transmiting any pending output
 */
void OBUF_startio()
{
    if(rgl.ready_bufs!=0)	/*if a buffer already being transmited*/
	return;			/*don't finish off this one*/
    OBUF_flush();
}

/*
 * drop off a buffer, don't really do anything
 * wait for a flush
 */
void PF_dropoff(m)
PFM_pt m;
{
    int len=BYTES_IN_BUF(*m);
    PFM_PUT16(m->initial,len);
    rgl.ins.buf+=len;
}

/*
 * return the number of free output buffers
 */
int OBUF_remaining()
{
    return NUM_OBUFS-rgl.ready_bufs;
}

/*
 * return an output buffer for our caller to fill in
 * then the caller drops it off with OBUF_dropoff()
 */
void PF_aloc(m,asize)
PFM_pt m;
long asize;
{
    if(asize>MAX_PAK_SIZE)
	rl_error("wanted too big of a output buffer");
    if((rgl.ins.buf!=NIL)&&
	(PFM_remaining(rgl.ins)<asize))
	OBUF_flush();

    if(rgl.ins.buf==NIL) {
	if(rgl.ready_bufs>=NUM_OBUFS)
	    rl_error("out of output buffers");
	rgl.ins.initial=(uint8 *)rgl.bufs + WRAP_OINDEX(rgl.insi) * MAX_PAK_SIZE;
	rgl.ins.buf=rgl.ins.initial+2;
	rgl.ins.limit=rgl.ins.initial+MAX_PAK_SIZE;
    }

    *m=rgl.ins;
    m->initial=m->buf;		/*it doesn't see the previous guys data*/
}

/***********************************************************************
 *								       *
 *                remote list management			       *
 *								       *
 ***********************************************************************/




void sysm_open();
void sysm_hello();
void sysm_size();
void sysm_newpc();
void sysm_urgent();
void sysm_urgent_reply();
void sysm_file_ops();
void sysm_alert();

static RL_rtn_pt sysmtab[NUM_SYSM]={
    sysm_open,			/*SYSM_open*/
    sysm_hello,			/*SYSM_hello*/
    sysm_size,			/*SM2C_size*/
    sysm_newpc,			/*SM2C_newpc*/
    sysm_urgent,		/*SM2C_urgent for confirm user */
    sysm_urgent_reply,		/* urgent reply to sysm_urgent */
    sysm_file_ops,		/* file operation routine */
    sysm_alert,			/* alert messages */
};

/*
 * read a system messages
 */
void rl_read_sysm(li,m,mi)
RL_header_pt li;
PFM_pt m;
int mi;
{
    if((mi<0) || (mi>=NUM_SYSM)) rl_error("system error");
/*    if (we_are_urgent() && mi != SYSM_urgent_reply) return; */
    (*(sysmtab[mi]))(li,m,mi);
}





/*
 * process all the incomming messages
 */
int  RL_process_incomming()
{
    PFM cur;
    int result=FALSE;
    while(IBUF_feed_me(&cur)) {
      (*rl_incomming_func)(&cur);
      result=TRUE;
    }
    return result;
}

